import nonebot
from nonebot.log import logger
from nonebot.plugin import export

from .browser import get_browser, shutdown_browser, get_new_page
from .date_source import text_to_pic, md_to_pic, template_to_html, html_to_pic

driver = nonebot.get_driver()
config = driver.config
export = export()


@driver.on_startup
async def init(**kwargs):
    """Start Browser

    Returns:
        Browser: Browser
    """
    browser = await get_browser(**kwargs)
    logger.info("Browser Started.")
    return browser


@driver.on_shutdown
async def shutdown():
    await shutdown_browser()
    logger.info("Browser Stoped.")


export.browser = init
export.text_to_pic = text_to_pic
export.get_new_page = get_new_page
export.md_to_pic = md_to_pic
export.template_to_html = template_to_html
export.html_to_pic = html_to_pic



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@Author         : yanyongyu
@Date           : 2021-03-12 13:42:43
@LastEditors    : yanyongyu
@LastEditTime   : 2021-11-01 14:05:41
@Description    : None
@GitHub         : https://github.com/yanyongyu
"""
__author__ = "yanyongyu"

from contextlib import asynccontextmanager
from typing import Optional, AsyncIterator
from nonebot.log import logger
from . import patch

from playwright.async_api import Page, Browser, async_playwright, Error

_browser: Optional[Browser] = None
_playwright = None

async def init(**kwargs) -> Browser:
    global _browser
    global _playwright
    _playwright = await async_playwright().start()
    try:
        _browser = await launch_browser(**kwargs)
    except Error:
        await install_browser()
        _browser = await launch_browser(**kwargs)
    return _browser


async def launch_browser(**kwargs) -> Browser:
    return await _playwright.chromium.launch(**kwargs)


async def get_browser(**kwargs) -> Browser:
    return _browser or await init(**kwargs)


@asynccontextmanager
async def get_new_page(**kwargs) -> AsyncIterator[Page]:
    browser = await get_browser()
    page = await browser.new_page(**kwargs)
    try:
        yield page
    finally:
        await page.close()


async def shutdown_browser():
    await _browser.close()
    await _playwright.stop()


async def install_browser():
    logger.info("正在安装 chromium")
    import sys
    from playwright.__main__ import main
    sys.argv = ['', 'install', 'chromium']
    try:
        main()
    except SystemExit:
        pass



from jinja2.environment import Template
from .browser import get_new_page
from pathlib import Path
import jinja2
from nonebot.log import logger
import markdown
import aiofiles

TEMPLATES_PATH = str(Path(__file__).parent / "templates")

env = jinja2.Environment(
    extensions=["jinja2.ext.loopcontrols"],
    loader=jinja2.FileSystemLoader(TEMPLATES_PATH),
    enable_async=True,
)


async def text_to_pic(text: str, css_path: str = None, width: int = 500) -> bytes:
    """多行文本转图片

    Args:
        text (str): 纯文本, 可多行
        css_path (str, optional): css文件
        width (int, optional): 图片宽度，默认为 500

    Returns:
        bytes: 图片, 可直接发送
    """
    template = env.get_template("text.html")

    return await html_to_pic(
        await template.render_async(
            text=text,
            css=await read_file(css_path) if css_path else await read_tpl("text.css"),
        ),
        viewport={"width": width, "height": 10},
    )


async def md_to_pic(
    md: str = None, md_path: str = None, css_path: str = None, width: int = 500
) -> bytes:
    """markdown 转 图片

    Args:
        md (str, optional): markdown 格式文本
        md_path (str, optional): markdown 文件路径
        css_path (str,  optional): css文件路径. Defaults to None.
        width (int, optional): 图片宽度，默认为 500

    Returns:
        bytes: 图片, 可直接发送
    """
    template = env.get_template("markdown.html")
    if not md:
        if md_path:
            md = await read_file(md_path)
        else:
            raise "必须输入 md 或 md_path"
    logger.debug(md)
    md = markdown.markdown(
        md,
        extensions=[
            "pymdownx.tasklist",
            "tables",
            "fenced_code",
            "codehilite",
            "mdx_math",
        ],
        extension_configs={"mdx_math": {"enable_dollar_delimiter": True}},
    )

    logger.debug(md)
    extra = ""
    if "math/tex" in md:
        katex_css = await read_tpl("katex/katex.min.b64_fonts.css")
        katex_js = await read_tpl("katex/katex.min.js")
        mathtex_js = await read_tpl("katex/mathtex-script-type.min.js")
        extra = (
            f'<style type="text/css">{katex_css}</style>'
            f"<script defer>{katex_js}</script>"
            f"<script defer>{mathtex_js}</script>"
        )

    if css_path:
        css = await read_file(css_path)
    else:
        css = await read_tpl("github-markdown-light.css") + await read_tpl(
            "pygments-default.css"
        )

    return await html_to_pic(
        await template.render_async(md=md, css=css, extra=extra),
        viewport={"width": width, "height": 10},
    )


# async def read_md(md_path: str) -> str:
#     async with aiofiles.open(str(Path(md_path).resolve()), mode="r") as f:
#         md = await f.read()
#     return markdown.markdown(md)


async def read_file(path: str) -> str:
    async with aiofiles.open(path, mode="r") as f:
        return await f.read()


async def read_tpl(path: str) -> str:
    return await read_file(f"{TEMPLATES_PATH}/{path}")


async def template_to_html(
    template_path: str,
    template_name: str,
    **kwargs,
) -> str:
    """使用jinja2模板引擎通过html生成图片

    Args:
        template_path (str): 模板路径
        template_name (str): 模板名
        **kwargs: 模板内容
    Returns:
        str: html
    """

    template_env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(template_path),
        enable_async=True,
    )
    template = template_env.get_template(template_name)

    return await template.render_async(**kwargs)


async def html_to_pic(html: str, wait: int = 0, **kwargs) -> bytes:
    """html转图片

    Args:
        html (str): html文本
        wait (int, optional): 等待时间. Defaults to 0.

    Returns:
        bytes: 图片, 可直接发送
    """
    # logger.debug(f"html:\n{html}")
    async with get_new_page(**kwargs) as page:
        await page.set_content(html, wait_until="networkidle")
        await page.wait_for_timeout(wait)
        img_raw = await page.screenshot(full_page=True)
    return img_raw



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_htmlrender']

package_data = \
{'': ['*'], 'nonebot_plugin_htmlrender': ['templates/*', 'templates/katex/*']}

install_requires = \
['Pygments>=2.10.0,<3.0.0',
 'jinja2>=3.0.3,<4.0.0',
 'markdown>=3.3.6,<4.0.0',
 'nonebot-adapter-cqhttp>=2.0.0-alpha.16,<3.0.0',
 'nonebot2>=2.0.0-alpha.16,<3.0.0',
 'playwright>=1.17.2,<2.0.0',
 'pymdown-extensions>=9.1,<10.0',
 'python-markdown-math>=0.8,<0.9']

setup_kwargs = {
    'name': 'nonebot-plugin-htmlrender',
    'version': '0.0.3',
    'description': '',
    'long_description': '# nonebot-plugin-htmlrender\n\n* 通过浏览器渲染图片\n* 可通过查看`example`参考使用实例\n\n# ✨ 功能\n\n* 通过html和浏览器生成图片\n* 支持`纯文本` `markdown` 和 `jinja2` 模板输入 \n* 通过 CSS 来控制样式\n\n\n\n# 使用\n\n参考[example/plugins/render/__init__.py](example/plugins/render/__init__.py)\n\n## markdown 转 图片\n\n- 使用 `GitHub-light` 样式\n- 支持绝大部分 md 语法\n- 代码高亮\n- latex 数学公式 （感谢@[MeetWq](https://github.com/MeetWq)）\n    - 使用 `$$...$$` 来输入独立公式\n    - 使用 `$...$` 来输入行内公式\n- 图片需要使用外部连接并使用`html`格式 否则文末会超出截图范围\n\n# 🌰 栗子\n\n[example.md](docs/example.md)\n## 文本转图片（同时文本里面可以包括html图片）\n![](docs/text2pic.png)\n\n## markdown转图片（同时文本里面可以包括html图片）\n![](docs/md2pic.png)\n\n## 纯html转图片\n![](docs/html2pic.png)\n\n## jinja2模板转图片\n![](docs/template2pic2pic.png)\n\n\n# 特别感谢\n\n- [MeetWq](https://github.com/MeetWq) 提供数学公式支持代码和代码高亮',
    'author': 'kexue',
    'author_email': 'xana278@foxmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
